#include <stdio.h> 
#include <stdlib.h> 
#include <errno.h> 
#include <string.h> 
#include <sys/socket.h> 
#include <netinet/in.h>
#include <netdb.h> 
#include <signal.h>
#include <time.h>
#include <sys/timex.h>

#include "common.h"

volatile sig_atomic_t done;

void inthandler(int Sig) { /*   INT -   Ctrl+C */
	done = 1;
}

int
main(int argc, char *argv[])
{
	int sockfd=0;  
	char data[1]={23};
	struct hostent *he;
	struct sockaddr_in their_addr; 

	if (argc < 2) {
		fprintf(stderr,"usage: %s hostname\n", argv[0]);
		return EXIT_FAILURE; 
	}

	if ((he=gethostbyname(argv[1])) == NULL) {  
		perror("gethostbyname");
		return EXIT_FAILURE; 
	}

	if ((sockfd = socket(AF_INET, SOCK_STREAM, 0)) == -1) {
		perror("socket");
		return EXIT_FAILURE; 
	}

	their_addr.sin_family = AF_INET;   
	their_addr.sin_port = htons(SERVER_PORT);
	their_addr.sin_addr = *((struct in_addr *)he->h_addr);

	memset(their_addr.sin_zero, 0, sizeof their_addr.sin_zero); 

	if (connect(sockfd, (struct sockaddr *)&their_addr, sizeof(struct sockaddr)) ==-1) {
		perror("connect");
		return EXIT_FAILURE; 
	}

	if(signal(SIGINT, SIG_IGN) != SIG_IGN)
		signal(SIGINT, inthandler);
	if(signal(SIGINT, SIG_IGN) != SIG_IGN)
		signal(SIGINT, inthandler);

    if (send(sockfd, data, 1, 0) == -1) {
		perror("send");
		return EXIT_FAILURE; 
	}

	memset(data, 0, sizeof data);

	fprintf(stderr, "Receiving data, press Ctrl+C to interrupt...\n");
	unsigned long cnt = 0; /* ޣ   */
	struct data_entry *de = NULL; /*    ɣ */ 
	de = (struct data_entry *) malloc(sizeof(struct data_entry)); /*     ɣ... */
	memset(de, 0, sizeof(struct data_entry)); /* ...  ţ  */

	struct ntptimeval t_start, t_end; /*      */

	/* ɣ    */
	if (ntp_gettime(&t_start)) {
		perror("ntp_gettime");
		return EXIT_FAILURE; 
	}
	while(!done) {
		if (recv(sockfd, de, sizeof(struct data_entry), 0) == -1) {
			perror("recv");
			return EXIT_FAILURE; 
		}
		cnt++;
	}
	if (ntp_gettime(&t_end)) {
		perror("ntp_gettime");
		return EXIT_FAILURE; 
	}

	int de_size = sizeof(struct data_entry);
	double seconds_elapsed = get_time_diff(&t_start, &t_end); /*  , . */
	double bandwith = ((cnt * de_size) / seconds_elapsed) / (1024 * 1024); /*  , /. */
	
	fprintf(stderr, "%d blocks received (%d bytes each block) in %.3f seconds\n", cnt, de_size, seconds_elapsed);
	fprintf(stderr, "Speed: %.2f MB/sec.\n", bandwith);
	free(de);
	
	shutdown(sockfd,2); /*   */
	return EXIT_SUCCESS; 
}

double get_time_diff(struct ntptimeval *start, struct ntptimeval *end) {
	return get_utime(end) - get_utime(start);
}

double get_utime(struct ntptimeval *t) {
	return ((double)t->time.tv_sec + (double)t->time.tv_usec / (double)1000000);
}
