#include <stdio.h>
#include <stdlib.h>
#include <errno.h>
#include <unistd.h>
#include <sys/types.h>
#include <sys/socket.h>
#include <netinet/in.h>
#include <netdb.h>
#include <signal.h>

#include "common.h"

struct data_entry *de = NULL;

int
main (void)
{
	int sock; /* server socket, which accepts new connections */
	fd_set active_fd_set, read_fd_set; /* sets of serving socket descriptors */
	int i;
	struct sockaddr_in clientname, server;
	size_t size;

	/* Create the socket and set it up to accept connections */
	sock = socket(PF_INET, SOCK_STREAM, 0);
	if (sock < 0) {
		perror("socket");
		exit(EXIT_FAILURE);
	}

	/*  ,     */
	de = (struct data_entry *) malloc(sizeof(struct data_entry));
	memset(de, 0, sizeof(struct data_entry));
	int k;
	for (k = 0; k++; k < QUANTITY)
		de->discretes[k] = 253;

	/* Give the socket a name */
	server.sin_family = AF_INET;
	server.sin_port = htons(SERVER_PORT);
	server.sin_addr.s_addr = htonl(INADDR_ANY);

	if (bind(sock, (struct sockaddr *) &server, sizeof(server)) < 0) {
		perror("bind");
		exit(EXIT_FAILURE);
	}

	if (listen(sock, 10) < 0) {
		perror("listen");
		exit(EXIT_FAILURE);
	}

	/* Initialize the set of active sockets */
	FD_ZERO (&active_fd_set);
	FD_SET (sock, &active_fd_set);

	signal(SIGPIPE, SIG_IGN);

	/*    */
	while (1) {
		/*   -    ,          */
		read_fd_set = active_fd_set;
		if (select (FD_SETSIZE, &read_fd_set, NULL, NULL, NULL) < 0) {
			perror ("select");
			exit (EXIT_FAILURE);
		}

		/*        */
		for (i = 0; i < FD_SETSIZE; ++i) {
			if (!FD_ISSET(i, &read_fd_set))
				continue; /*     ,   */
				
			if (i == sock) {
				/*     -        */
				int new;
				size = sizeof (clientname);

				/*      */
				new = accept(sock, (struct sockaddr *) &clientname, &size);
				if (new < 0) {
					perror ("accept");
					exit (EXIT_FAILURE);
				}

				/*   ,         */

				fprintf (stderr, "connect from host %s, port %hd.\n",
					inet_ntoa (clientname.sin_addr),
					ntohs(clientname.sin_port));
								
				/*         - select */
				FD_SET (new, &active_fd_set);
			}
			else {
				/*    */
				if (serve_client (i) < 0) {
					close (i);
					FD_CLR (i, &active_fd_set);
					fprintf (stderr, "client disconnected\n\n");
				}
			}
		}
	}
}


/*   */
int
serve_client (int fd)
{
	char buffer[2];
	int nbytes, mode;

	nbytes = read (fd, buffer, 1); /*     -   */
	if (nbytes < 0) {
		/*   */
		perror("read");
		exit(EXIT_FAILURE);
	}
	else if (nbytes == 0) {
		/*   */
		return -1;
	}
	else {
		mode = buffer[0];
		fprintf (stderr, "switching to %d mode\n", mode);
	}

	fprintf(stderr, "Making service...\n");
	make_service(fd, mode);
	return -1; /*    */	
}

/*     fd    mode */
void make_service(int fd, int mode) {
	if (mode != 23) {
		fprintf (stderr, "Unsupported mode: %d\n", mode);
	}
	else {
		int done = 0;
		unsigned long cnt = 0;
		int nbytes = 0;

		while (!done) {
			nbytes = send(fd, de, sizeof(struct data_entry), 0);
			if (nbytes < 1 && errno == EPIPE) {
				done = 1;
			}
			else if (nbytes < 1) {
				perror("send2");
				exit(EXIT_FAILURE);
			}
			else {
				cnt++;
			}
		}
		int size = sizeof(struct data_entry);
		fprintf(stderr, "%d blocks (%d each block) was transmitted to client\n", cnt, size);
	}
}

/*      
void * calculations (int mode) {
	return de;
}
*/
